// adv2asc.c : This file contains the 'main' function. Program execution begins and ends there.
//

// $Date: 2019-02-27 11:15:25 +0100 (Wed, 27 Feb 2019) $
// $Revision: 68740 $
// $Author: jhuizinga $
// $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/bin2asc/sourceADV/adv2asc.c $

// pch.h uitgezet vanwege compileren onder lcc
//#include "pch.h"
#include "adv2asc.h"
#include <stdio.h>
#include <stdlib.h>

int main(int argc, char **argv)
{

  FILE *          infile;
  FILE *          outfile;
  FILE *          logfile;

  unsigned char   cSync;
  unsigned char   cId;

  PdVecVel        pdVecVel;
  PdVecSys        pdVecSys;
  int             j;
  int             nBeams = 3;
  errno_t         err;
  char *          bError;
  char *          bStatus;
  long            positie = 0;
  unsigned short  hChecksum;
  unsigned short  nChecksum = 0;
  size_t          nElements;

  if (argc != 4)
  {
    printf("Usage: adv2asc infile outfile logfile");
    exit(1);
  }

  // Open file's
  err = fopen_s(&logfile, argv[3], "a");
  if (err != 0) {
    fprintf(logfile, "Error : opening logfile file %s\n", argv[3]);
    exit(1);
  }
  err = fopen_s(&infile, argv[1], "rb");
  if (err != 0) {
    fprintf(logfile, "Error : opening input file %s\n", argv[1]);
    exit(1);
  }
  err = fopen_s(&outfile, argv[2], "w");
  if (err != 0) {
    fprintf(logfile, "Error : opening output file %s\n", argv[2]);
    exit(1);
  }

  fprintf(logfile, "Conversie naar ascii van %s naar %s gestart!\n", argv[1], argv[2]);
  //
  // Eenmalig weergeven wat filenaam is
  fprintf(outfile, "Filename: %s \n", argv[1]);
  fprintf(outfile, "Conversie naar ascii: %s \n", "ADV2ASC_$Revision: 68740 $");

  fseek(infile, 0L, 0);

  while (!feof(infile)) {
    cId = 0;

    fread(&cSync, sizeof(cSync), 1, infile);
    fread(&cId, sizeof(cId), 1, infile);

    if (cSync == 165 && cId == 16) {

      positie = (ftell(infile) - 2L);
      fseek(infile, positie, 0);

      // Vector velocity data 3 beams
      nElements = fread(&pdVecVel, sizeof(pdVecVel), 1, infile);
      if (nElements > 0) {

        // Checksum control
        hChecksum = Checksum((short *)&pdVecVel, (sizeof(pdVecVel) / 2) - 1);
        if (pdVecVel.hChecksum != hChecksum) {
          // Handle the error.
          nChecksum++;
          fprintf(logfile, "Checksum error %d != %d\n", pdVecVel.hChecksum, hChecksum);
          fseek(infile, 0L, SEEK_END);
        }
        else {
          fprintf(outfile, "sync," "identification," "cAnaIn2LSB," "cCount," "pressureMSB," "pressureLSW\n");

          fprintf(outfile, "%x,%x,%d,%d,%d,%d\n",
            pdVecVel.cSync,
            pdVecVel.cId,
            pdVecVel.cAnaIn2LSB,
            pdVecVel.cCount,
            pdVecVel.cPressureMSB,
            pdVecVel.hPressureLSW);

          fprintf(outfile, "hVel\n");
          for (j = 0; j < nBeams; j++) {
            fprintf(outfile, "%d,", pdVecVel.hVel[j]);
          }
          fprintf(outfile, "\n");

          fprintf(outfile, "cAmp\n");
          for (j = 0; j < nBeams; j++) {
            fprintf(outfile, "%d,", pdVecVel.cAmp[j]);
          }
          fprintf(outfile, "\n");

          fprintf(outfile, "cCorr\n");
          for (j = 0; j < nBeams; j++) {
            fprintf(outfile, "%d,", pdVecVel.cCorr[j]);
          }
          fprintf(outfile, "\n");
        }
      }
    }
    else if (cSync == 165 && cId == 17) {

      positie = (ftell(infile) - 2L);
      fseek(infile, positie, 0);

      nElements = fread(&pdVecSys, sizeof(pdVecSys), 1, infile);
      if (nElements > 0) {
        // Checksum control
        hChecksum = Checksum((short *)&pdVecSys, (sizeof(pdVecSys) / 2) - 1);
        if (pdVecSys.hChecksum != hChecksum) {
          // Handle the error.
          nChecksum++;
          fprintf(logfile, "Checksum error pdVecSys %d != %d\n", pdVecVel.hChecksum, hChecksum);
          fseek(infile, 0L, SEEK_END);
        }
        else {

          fprintf(outfile,
            "sync,"
            "identification,"
            "structuresize,"
            "year,"
            "month,"
            "day,"
            "hours,"
            "minutes,"
            "seconds,"
            "battery_voltage(0.1V),"
            "soundspeed(0.1m/s),"
            "hHeading(0.1deg),"
            "hPitch(0.1deg),"
            "hRoll(0.1deg),"
            "temperature(0.01 degC),"
            "ErrorCode,"
            "Status,"
            "analogInput,"
            "Checksum \n");

          bError = decimal_to_binary(pdVecSys.cError);
          bStatus = decimal_to_binary(pdVecSys.cStatus);

          //printf("error : %d %s, status : %d %s\n", pdVecSys.cError, bError, pdVecSys.cStatus, bStatus);

          fprintf(outfile, "%x,%x,%d,%x,%x,%x,%x,%x,%x,%d,%d,%d,%d,%d,%d,%s,%s,%d,%d\n",
            pdVecSys.cSync,
            pdVecSys.cId,
            pdVecSys.hSize,
            pdVecSys.clock.cYear,
            pdVecSys.clock.cMonth,
            pdVecSys.clock.cDay,
            pdVecSys.clock.cHour,
            pdVecSys.clock.cMinute,
            pdVecSys.clock.cSecond,
            pdVecSys.hBattery,
            pdVecSys.hSoundSpeed,
            pdVecSys.hHeading,
            pdVecSys.hPitch,
            pdVecSys.hRoll,
            pdVecSys.hTemperature,
            bError,
            bStatus,
            pdVecSys.hAnaIn,
            pdVecSys.hChecksum);

          free(bError);
          free(bStatus);
        }
      }
    }
  }

  fclose(infile);
  fclose(outfile);
  fprintf(logfile, "Number of checksum errors = %d\n", nChecksum);
  fprintf(logfile, "Conversie naar ascii van %s naar %s einde!\n", argv[1], argv[2]);
  fclose(logfile);

   exit(0);
}

char *decimal_to_binary(char n)
{
  int c, d, count;
  char *pointer;

  count = 0;
  pointer = (char*)malloc(8 + 1);

  if (pointer == NULL)
    exit(EXIT_FAILURE);

  for (c = 7; c >= 0; c--)
  {
    d = n >> c;

    if (d & 1)
      *(pointer + count) = 1 + '0';
    else
      *(pointer + count) = 0 + '0';

    count++;
  }
  *(pointer + count) = '\0';

  return  pointer;
}

unsigned short Checksum(short *phBuff, int n) {
  int i;

  unsigned short hChecksum = 0xb58c;

  for (i = 0; i < n; i++)
    hChecksum += phBuff[i];
  return hChecksum;
}
